//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_SYSLOG_H
#define LIBUNCIA_OUTPUT_SYSLOG_H

#include <libuncia/ostream.h>
#include <libuncia/output.h>

namespace libuncia
{
    /**
      * The output_syslog class is used to represent an output sink
      * which writes its text to syslog(3).
      *
      * The <i>ident</i> passed to <i>openlog</i>(3) is the program
      * name, as returned by #progname_get
      */
    class output_syslog:
        public output
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_syslog();

    private:
        /**
          * The default constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param facility
          *     The facility argument is used to specify what type
          *     of program is logging the message.  This lets the
          *     configuration file specify that messages from different
          *     facilities will be handled differently.
          *     Defaults to LOG_USER if not specified.
          * @param level
          *     This determines the importance of the messages.
          *     Defaults to LOG_ERR if not specified.
          * @param option
          *     Defaults to 0 (no flags) if not specified.
          */
        output_syslog(int facility = -1, int level = -1, int option = -1);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param facility
          *     The facility argument is used to specify what type
          *     of program is logging the message.  This lets the
          *     configuration file specify that messages from different
          *     facilities will be handled differently.
          *     Defaults to LOG_USER if not specified.
          * @param level
          *     This determines the importance of the messages.
          *     Defaults to LOG_ERR if not specified.
          * @param option
          *     Defaults to 0 (no flags) if not specified.
          */
        static pointer create(int facility = -1, int level = -1,
            int option = -1);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

        // See base class for documentation.
        int sync();

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The singleton class variable is used to track the one and
          * only instance to be created.  It is managed by the #create
          * class method.
          */
        static pointer singleton;

        /**
          * The facility instance variable is used to remember what type
          * of program is logging the message
          */
        int facility;

        /**
          * The level instance variable is used to remember the
          * importance of the messages.
          */
        int level;

        /**
          * The line instance variable is used to rememebr the current
          * line being constructed.  This allows <i>syslog</i>(3) to be
          * called once per line, even when lines span #write calls.
          */
        std::string line;

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_syslog(const output_syslog &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_syslog &operator=(const output_syslog &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_SYSLOG_H
